/* This file is part of flowz.
   Copyright (C) 2006 Sergey Poznyakoff

   Flowz is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   Flowz is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with flowz; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
   MA 02110-1301 USA */

#include "flowz.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define MAKE_NET(ipstr,lenstr,buf) do {                                       \
     char *p;                                                                 \
     unsigned long ip;                                                        \
     struct in_addr addr;                                                     \
     unsigned len;                                                            \
                                                                              \
     if (inet_aton(ipstr, &addr) == 0) {                                      \
	  error(0, 0, "%s:%lu: invalid IP address %s",                        \
		input, (unsigned long) lineno, ipstr);                        \
	  return 1;                                                           \
     }                                                                        \
                                                                              \
     len = strtoul(lenstr, &p, 0);                                            \
     if (*p) {                                                                \
	  error(0, 0,                                                         \
		"%s:%lu: cannot convert source netmask length (near %s)",     \
		input, (unsigned long) lineno, p);                            \
	  return 1;                                                           \
     }                                                                        \
     if (len > 32) {                                                          \
	  error(0, 0, "%s:%lu: invalid source netmask length %s",             \
		input, (unsigned long) lineno, lenstr);                       \
	  return 1;                                                           \
     }                                                                        \
                                                                              \
     len = 32 - len;                                                          \
     addr.s_addr = htonl(ntohl(addr.s_addr) & (0xfffffffful >> len) << len);  \
                                                                              \
     ipstr = strcpy(buf, inet_ntoa(addr));                                    \
} while (0)

int
parse_timerange(const char *str, time_t *ptime_start, time_t *ptime_end)
{
     uintmax_t n;
     char *p;
     time_t timestamp;
     
     timestamp = n = strtoumax(str, &p, 0);
     if (*p != '-') {
	  error(0, 0, "%s:%lu: cannot convert start time (near %s)",
		input, (unsigned long) lineno, p);
	  return 1;
     } else if (n != timestamp || errno == ERANGE) {
	  error(0, 0, "%s:%lu: cannot convert start time (out of allowed range)",
		input, (unsigned long) lineno);
	  return 1;
     }

     *ptime_start = timestamp;
     
     /* *** */

     timestamp = n = strtoumax(p+1, &p, 0);
     if (*p) {
	  error(0, 0, "%s:%lu: cannot convert end time (near %s)",
		input, (unsigned long) lineno, p);
	  return 1;
     } else if (n != timestamp || errno == ERANGE) {
	  error(0, 0, "%s:%lu: cannot convert end time (out of allowed range)",
		input, (unsigned long) lineno);
	  return 1;
     }
     
     *ptime_end = timestamp;
     return 0;
}

int
level_2(char **toks, size_t ntoks)
{
     time_t time_start;
     time_t time_end;
     uintmax_t octets;
     uintmax_t n;
     char *p;
     char srcbuf[16], dstbuf[16];
     static unsigned char keymap[] = { 2, 3, 4, 5, 6, 7, 8, 9, 10 };
     
     if (parse_timerange(toks[0], &time_start, &time_end))
	  return 1;

     /* *** */
     
     octets = strtoumax(toks[1], &p, 0);
     if (*p) {
	  error(0, 0, "%s:%lu: cannot convert octets (near %s)",
		input, (unsigned long) lineno, p);
	  return 1;
     } else if (errno == ERANGE) {
	  error(0, 0, "%s:%lu: cannot convert octets (out of allowed range)",
		input, (unsigned long) lineno);
	  return 1;
     }

     /* Make source net */
     MAKE_NET(toks[2], toks[5], srcbuf);

     /* Make destination net */
     MAKE_NET(toks[3], toks[6], dstbuf);

     if (update(make_key(keymap, sizeof keymap/sizeof keymap[0], toks),
		time_start, time_end, octets))
	  error(0, 0, "integer overflow");
     return 0;
}

